"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
exports.__esModule = true;
var renamer_1 = require("./renamer");
var namespec_1 = require("./namespec");
var incrementer_1 = require("./incrementer");
var log_1 = require("./log");
var NAMESPEC = '.namespec';
// Adapted from https://stackoverflow.com/a/34509653
function ensureDirectoryExistence(filePath, dirnameFn, fs) {
    var dirname = dirnameFn(filePath);
    if (fs.existsSync(dirname))
        return;
    ensureDirectoryExistence(dirname, dirnameFn, fs);
    fs.mkdirSync(dirname);
}
function getAllMatches(regex, str) {
    var result;
    var results = [];
    while ((result = regex.exec(str)) !== null) {
        results.push(result);
    }
    return results;
}
var incrementers = {
    minimal: incrementer_1.MinimalIncrementer,
    simple: incrementer_1.SimpleIncrementer,
    module: incrementer_1.ModuleIncrementer
};
var Distinguisher = /** @class */ (function () {
    function Distinguisher(distinguishConfig, fs, dirnameFn, logging) {
        if (logging === void 0) { logging = true; }
        this.distinguishConfig = distinguishConfig;
        this.fs = fs;
        this.dirnameFn = dirnameFn;
        this.logging = logging;
        var incrementer = incrementers[distinguishConfig.incrementer];
        this.rootRenamer = new renamer_1.Renamer(incrementer, distinguishConfig.types);
    }
    Distinguisher.prototype.walkSync = function (dir, outDir, renamer, filelist) {
        var _this = this;
        if (dir === void 0) { dir = ''; }
        if (outDir === void 0) { outDir = ''; }
        if (filelist === void 0) { filelist = []; }
        var e_1, _a, e_2, _b, e_3, _c, e_4, _d, e_5, _e, e_6, _f, e_7, _g;
        if (!dir.endsWith('/'))
            dir += '/';
        if (!outDir.endsWith('/'))
            outDir += '/';
        var namespecPath = "" + dir + NAMESPEC;
        if (this.fs.existsSync(namespecPath)) {
            // Parse a namespec file to determine the renamer's new scope.
            var namespec = null;
            try {
                namespec = new namespec_1.NamespecParser(this.fs.readFileSync(namespecPath).toString()).parse();
            }
            catch (e) {
                log_1.logStyle(log_1.FAIL, "Error parsing " + namespecPath + ": " + e);
            }
            if (namespec != null) {
                // Set the namespace.
                renamer = renamer.namespace(renamer_1.Renamer.pathSpecToParts(namespec.namespace));
                try {
                    // Set imports.
                    for (var _h = __values(namespec.imports.entries()), _j = _h.next(); !_j.done; _j = _h.next()) {
                        var _k = __read(_j.value, 2), importName = _k[0], importMap = _k[1];
                        try {
                            for (var _l = __values(importMap.entries()), _m = _l.next(); !_m.done; _m = _l.next()) {
                                var _o = __read(_m.value, 2), type = _o[0], names = _o[1];
                                try {
                                    for (var names_1 = __values(names), names_1_1 = names_1.next(); !names_1_1.done; names_1_1 = names_1.next()) {
                                        var name_1 = names_1_1.value;
                                        renamer["import"](importName, type, name_1);
                                    }
                                }
                                catch (e_3_1) { e_3 = { error: e_3_1 }; }
                                finally {
                                    try {
                                        if (names_1_1 && !names_1_1.done && (_c = names_1["return"])) _c.call(names_1);
                                    }
                                    finally { if (e_3) throw e_3.error; }
                                }
                            }
                        }
                        catch (e_2_1) { e_2 = { error: e_2_1 }; }
                        finally {
                            try {
                                if (_m && !_m.done && (_b = _l["return"])) _b.call(_l);
                            }
                            finally { if (e_2) throw e_2.error; }
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (_j && !_j.done && (_a = _h["return"])) _a.call(_h);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                try {
                    // Set reserves.
                    for (var _p = __values(namespec.reserves.entries()), _q = _p.next(); !_q.done; _q = _p.next()) {
                        var _r = __read(_q.value, 2), type = _r[0], reserves = _r[1];
                        try {
                            for (var _s = __values(reserves.values()), _t = _s.next(); !_t.done; _t = _s.next()) {
                                var _u = __read(_t.value, 1), nameValue = _u[0];
                                renamer.reserve(type, nameValue);
                            }
                        }
                        catch (e_5_1) { e_5 = { error: e_5_1 }; }
                        finally {
                            try {
                                if (_t && !_t.done && (_e = _s["return"])) _e.call(_s);
                            }
                            finally { if (e_5) throw e_5.error; }
                        }
                    }
                }
                catch (e_4_1) { e_4 = { error: e_4_1 }; }
                finally {
                    try {
                        if (_q && !_q.done && (_d = _p["return"])) _d.call(_p);
                    }
                    finally { if (e_4) throw e_4.error; }
                }
                try {
                    // Set declares.
                    for (var _v = __values(Array.from(namespec.declares.entries())), _w = _v.next(); !_w.done; _w = _v.next()) {
                        var _x = __read(_w.value, 2), type = _x[0], declares = _x[1];
                        try {
                            for (var _y = __values(Array.from(declares.entries())), _z = _y.next(); !_z.done; _z = _y.next()) {
                                var _0 = __read(_z.value, 2), nameValue = _0[0], varValue = _0[1];
                                renamer.declare(type, nameValue, varValue);
                            }
                        }
                        catch (e_7_1) { e_7 = { error: e_7_1 }; }
                        finally {
                            try {
                                if (_z && !_z.done && (_g = _y["return"])) _g.call(_y);
                            }
                            finally { if (e_7) throw e_7.error; }
                        }
                    }
                }
                catch (e_6_1) { e_6 = { error: e_6_1 }; }
                finally {
                    try {
                        if (_w && !_w.done && (_f = _v["return"])) _f.call(_v);
                    }
                    finally { if (e_6) throw e_6.error; }
                }
            }
        }
        var files = this.fs.readdirSync(dir == '' ? '.' : dir);
        files.forEach(function (file) {
            var e_8, _a;
            var fn = "" + dir + file;
            try {
                for (var _b = __values(_this.distinguishConfig.exclude), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var exclude = _c.value;
                    if (fn.match(exclude) != null)
                        continue;
                }
            }
            catch (e_8_1) { e_8 = { error: e_8_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b["return"])) _a.call(_b);
                }
                finally { if (e_8) throw e_8.error; }
            }
            if (_this.fs.statSync(fn).isDirectory()) {
                filelist = _this.walkSync("" + (fn + (fn.endsWith('/') ? '' : '/')), "" + outDir + file, renamer, filelist);
            }
            else {
                filelist.push({
                    inputFile: "" + dir + file,
                    outputFile: "" + outDir + file,
                    renamer: renamer
                });
            }
        });
        return filelist;
    };
    Distinguisher.prototype.run = function () {
        var e_9, _a, e_10, _b, e_11, _c;
        var startTime = Date.now();
        try {
            for (var _d = __values(this.walkSync(this.distinguishConfig.inputDir, this.distinguishConfig.outputDir, this.rootRenamer)), _e = _d.next(); !_e.done; _e = _d.next()) {
                var _f = _e.value, inputFile = _f.inputFile, outputFile = _f.outputFile, renamer = _f.renamer;
                var contents = this.fs.readFileSync(inputFile).toString();
                var hadMatches = false;
                try {
                    for (var _g = __values(this.distinguishConfig.types), _h = _g.next(); !_h.done; _h = _g.next()) {
                        var type = _h.value;
                        // Find all matches.
                        var matches = getAllMatches(new RegExp("_(" + type + ")[$-]([a-zA-Z0-9_-]+)", 'g'), contents);
                        var offset = 0;
                        for (var i = 0; i < matches.length; i++) {
                            // Iterate in correct order and keep track of offset.
                            hadMatches = true; // there was at least a match somewhere
                            var _j = __read(matches[i], 3), fullMatch = _j[0], typeMatch = _j[1], name_2 = _j[2];
                            var index = matches[i].index;
                            index -= offset;
                            var renamed = renamer.addName(typeMatch, name_2);
                            offset += fullMatch.length - renamed.length;
                            contents =
                                contents.substr(0, index) +
                                    renamed +
                                    contents.substr(index + fullMatch.length);
                        }
                    }
                }
                catch (e_10_1) { e_10 = { error: e_10_1 }; }
                finally {
                    try {
                        if (_h && !_h.done && (_b = _g["return"])) _b.call(_g);
                    }
                    finally { if (e_10) throw e_10.error; }
                }
                if (this.logging && hadMatches) {
                    log_1.logStyle(log_1.STATUS, "Writing " + outputFile + " with namespace " + renamer.namespaces.join('/'));
                }
                ensureDirectoryExistence(outputFile, this.dirnameFn, this.fs);
                this.fs.writeFileSync(outputFile, contents.toString());
            }
        }
        catch (e_9_1) { e_9 = { error: e_9_1 }; }
        finally {
            try {
                if (_e && !_e.done && (_a = _d["return"])) _a.call(_d);
            }
            finally { if (e_9) throw e_9.error; }
        }
        var overallTime = Date.now() - startTime;
        if (this.logging)
            log_1.logStyle(log_1.BOLD, "\nWrote output in " + overallTime / 1000 + "s");
        var danglers = this.rootRenamer.danglingImports();
        if (this.logging && danglers.length > 0)
            console.log('\n');
        try {
            for (var danglers_1 = __values(danglers), danglers_1_1 = danglers_1.next(); !danglers_1_1.done; danglers_1_1 = danglers_1.next()) {
                var _k = danglers_1_1.value, sourceNamespace = _k.sourceNamespace, importNamespace = _k.importNamespace, type = _k.type, name_3 = _k.name;
                if (this.logging) {
                    log_1.logStyle(log_1.WARN, "Dangling import: " + sourceNamespace + " imports unused {type: " + type + ", name: " + name_3 + "} from " + importNamespace);
                }
            }
        }
        catch (e_11_1) { e_11 = { error: e_11_1 }; }
        finally {
            try {
                if (danglers_1_1 && !danglers_1_1.done && (_c = danglers_1["return"])) _c.call(danglers_1);
            }
            finally { if (e_11) throw e_11.error; }
        }
    };
    return Distinguisher;
}());
exports.Distinguisher = Distinguisher;
