"use strict";
/**
 * Parse files with contents like this:
 *
 *   namespace: component
 *
 *   from .. import
 *     css
 *       dog
 *       cat
 *       bark
 *     id
 *       yes
 *
 *   from /slider import
 *     css
 *       slider
 */
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
exports.__esModule = true;
function extractVarValue(nameVar) {
    var index = nameVar.indexOf('=');
    return [nameVar.substr(0, index), nameVar.substr(index + 1)];
}
var NamespecParser = /** @class */ (function () {
    function NamespecParser(specContents) {
        this.specContents = specContents;
    }
    NamespecParser.prototype.getLine = function () {
        if (this.specContents.length == 0)
            return { eof: true, length: 0 };
        var position = this.specContents.indexOf('\n');
        if (position == -1) {
            return { contents: this.specContents, length: this.specContents.length };
        }
        return { contents: this.specContents.substr(0, position), length: position + 1 };
    };
    NamespecParser.prototype.eof = function () {
        return this.specContents.length == 0;
    };
    NamespecParser.prototype.advance = function (n) {
        this.specContents = this.specContents.substr(n);
    };
    NamespecParser.prototype.consumeWhitespaceWhileExists = function () {
        while (true) {
            var line = this.getLine();
            if (line.contents == null)
                return;
            if (line.contents.match(/^\s*$/)) {
                this.advance(line.length);
            }
            else {
                break;
            }
        }
    };
    NamespecParser.prototype.consumeLine = function (spec, flag) {
        this.consumeWhitespaceWhileExists();
        var line = this.getLine();
        var length = line.length;
        // If at the end of file, return accordingly.
        if (line.eof != null)
            return { eof: true, length: length };
        var contents = line.contents;
        // Try to match the line.
        var match = contents.match(spec);
        // Return early if there is no match.
        if (match == null)
            return { match: false, length: length };
        // Return matching group one otherwise.
        this.advance(length);
        var result = { contents: match[1], allMatches: match, length: length };
        if (flag != null)
            result.flag = flag;
        return result;
    };
    NamespecParser.prototype.consumeNamespace = function () {
        var namespace = this.consumeLine(/^namespace (.+)$/);
        if (namespace.contents == null) {
            throw new Error('Expected to consume namespace');
        }
        return namespace.contents;
    };
    NamespecParser.prototype.consumeMethod = function () {
        var result = this.consumeLine(/^from ([^ ]+) import$/, 'import');
        if (result.contents == null) {
            result = this.consumeLine(/^(reserve)$/, 'reserve');
            if (result.contents == null) {
                result = this.consumeLine(/^(declare)$/, 'declare');
            }
        }
        return result;
    };
    NamespecParser.prototype.consumeIndented = function (expectedWhitespace, equalsOk) {
        if (equalsOk === void 0) { equalsOk = false; }
        var type;
        if (expectedWhitespace == null) {
            type = this.consumeLine(new RegExp("^([ \\t]+)([a-zA-Z0-9_-]+" + (equalsOk ? '=.*' : '') + ")$"));
        }
        else {
            type = this.consumeLine(new RegExp("^(" + expectedWhitespace + ")([a-zA-Z0-9_-]+" + (equalsOk ? '=.*' : '') + ")$"));
        }
        if (type.allMatches == null)
            return null;
        return {
            value: type.allMatches[2],
            whitespace: type.allMatches[1]
        };
    };
    NamespecParser.prototype.parse = function () {
        var _a;
        var namespace = this.consumeNamespace();
        var expectedTypeWhitespace = null;
        var expectedNameWhitespace = null;
        var imports = new Map();
        var reserves = new Map();
        var declares = new Map();
        var result = { namespace: namespace, imports: imports, reserves: reserves, declares: declares };
        while (true) {
            // Iterate through remaining file consuming import statements.
            var entryResult = this.consumeMethod();
            if (entryResult.contents == null) {
                // If at end-of-file, successful parse!
                this.consumeWhitespaceWhileExists();
                if (this.eof())
                    return result;
                // Throw consume error otherwise.
                throw new Error("Unexpected contents: " + this.specContents.substr(0, 15) + (this.specContents.length > 15 ? '...' : ''));
            }
            var importMap = null;
            var reservedMap = null;
            var declaredMap = null;
            if (entryResult.flag == 'import') {
                var importNamespace = entryResult.contents;
                if (!result.imports.has(importNamespace)) {
                    // Create the import map.
                    importMap = new Map();
                    result.imports.set(importNamespace, importMap);
                }
                else {
                    importMap = result.imports.get(importNamespace);
                }
            }
            else if (entryResult.flag != 'reserve' && entryResult.flag != 'declare') {
                throw new Error("Unexpected flag: " + entryResult.flag);
            }
            while (true) {
                // Iterate through import consuming top-level types.
                var type = this.consumeIndented(expectedTypeWhitespace);
                if (type == null)
                    break;
                var typeValue = type.value;
                expectedTypeWhitespace = type.whitespace;
                while (true) {
                    // Iterate through type consuming secondary-level names.
                    var name_1 = void 0;
                    if (expectedNameWhitespace == null) {
                        // If name indent level is unknown, search for more indented than type.
                        name_1 = this.consumeIndented(expectedTypeWhitespace + '[ \t]+', entryResult.flag == 'declare');
                    }
                    else {
                        name_1 = this.consumeIndented(expectedNameWhitespace, entryResult.flag == 'declare');
                    }
                    if (name_1 == null)
                        break;
                    var nameValue = name_1.value;
                    expectedNameWhitespace = name_1.whitespace;
                    // Add in the type, name pair.
                    if (importMap != null) {
                        // Add import
                        if (importMap.has(typeValue)) {
                            var nameEntries = importMap.get(typeValue);
                            nameEntries.push(nameValue);
                        }
                        else {
                            importMap.set(typeValue, [nameValue]);
                        }
                    }
                    else if (reserves != null) {
                        // Add reserved or declared.
                        var varValue = void 0;
                        if (entryResult.flag == 'declare') {
                            // Parse out declare value
                            _a = __read(extractVarValue(nameValue), 2), nameValue = _a[0], varValue = _a[1];
                            if (declares.has(typeValue)) {
                                var declaredMap_1 = declares.get(typeValue);
                                declaredMap_1.set(nameValue, varValue);
                            }
                            else {
                                declares.set(typeValue, new Map([[nameValue, varValue]]));
                            }
                        }
                        else {
                            if (reserves.has(typeValue)) {
                                var reservedSet = reserves.get(typeValue);
                                reservedSet.add(nameValue);
                            }
                            else {
                                reserves.set(typeValue, new Set([nameValue]));
                            }
                        }
                    }
                    else {
                        throw new Error('Expected import or reserved map to be set');
                    }
                }
            }
        }
    };
    return NamespecParser;
}());
exports.NamespecParser = NamespecParser;
