"use strict";
exports.__esModule = true;
var log_1 = require("./log");
function getKeys(object) {
    var results = [];
    for (var property in object) {
        if (object.hasOwnProperty(property))
            results.push(property);
    }
    return results.sort();
}
function getValues(object) {
    return getKeys(object).map(function (k) { return object[k]; });
}
var Tester = /** @class */ (function () {
    function Tester() {
    }
    Tester.prototype.test = function (name, fn) {
        try {
            fn.bind(this)();
            log_1.logStyle(log_1.PASS, name + " PASSED");
        }
        catch (e) {
            log_1.logStyle(log_1.FAIL, name + " FAILED:\n  " + e);
            process.on('exit', function () { return (process.exitCode = 1); });
        }
    };
    Tester.getType = function (obj) {
        if (Array.isArray(obj)) {
            return 'array';
        }
        else if (obj instanceof RegExp) {
            return 're';
        }
        else if (obj.constructor == Object) {
            return 'object';
        }
        else {
            return 'primitive';
        }
    };
    Tester.prototype.assertEquals = function (arg1, arg2) {
        var type1 = Tester.getType(arg1);
        var type2 = Tester.getType(arg2);
        if (type1 != type2) {
            throw new Error("Types of " + arg1 + " and " + arg2 + " are not the same: " + type1 + " vs. " + type2);
        }
        if (type1 == 'array') {
            this.assertArrayEquals(arg1, arg2);
        }
        else if (type1 == 'object') {
            this.assertObjectEquals(arg1, arg2);
        }
        else if (type1 == 're') {
            this.assertEquals(arg1.source, arg2.source);
        }
        else {
            if (arg1 != arg2)
                throw new Error("Expected " + arg1 + " to equal " + arg2);
        }
    };
    Tester.prototype.assertArrayEquals = function (arg1, arg2) {
        if (arg1.length != arg2.length) {
            throw new Error(arg1 + " and " + arg2 + " have differing lengths (" + arg1.length + " vs. " + arg2.length + ")");
        }
        for (var i = 0; i < arg1.length; i++) {
            this.assertEquals(arg1[i], arg2[i]);
        }
    };
    Tester.prototype.assertObjectEquals = function (arg1, arg2) {
        var properties1 = getKeys(arg1);
        var properties2 = getKeys(arg2);
        this.assertArrayEquals(properties1, properties2);
        var values1 = getValues(arg1);
        var values2 = getValues(arg2);
        this.assertArrayEquals(values1, values2);
    };
    Tester.prototype.assert = function (condition) {
        if (!condition)
            throw new Error("Expected " + condition + " to be truthy");
    };
    return Tester;
}());
exports.Tester = Tester;
