"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
exports.__esModule = true;
var renamer_1 = require("./renamer");
var tester_1 = require("./tester");
var namespec_1 = require("./namespec");
var incrementer_1 = require("./incrementer");
var virtual_fs_1 = require("./virtual-fs");
var distinguisher_1 = require("./distinguisher");
var cli_1 = require("./cli");
var t = new tester_1.Tester();
t.test('testBasicNaming', function () {
    var r = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['cls', 'id']);
    r.addName('cls', 'dog');
    r.addName('cls', 'cat');
    r.addName('cls', 'frog');
    var component = r.namespace('component1');
    component.addName('cls', 'machine');
    r.addName('cls', '1');
    component.addName('cls', 'dog');
    r.addName('cls', '2');
    r.addName('id', 'doggo');
    var rMap = r.namingMaps.get('cls');
    var rIdMap = r.namingMaps.get('id');
    var cMap = component.namingMaps.get('cls');
    t.assertEquals(rMap.get('dog'), 'a');
    t.assertEquals(rMap.get('cat'), 'b');
    t.assertEquals(rMap.get('frog'), 'c');
    t.assertEquals(cMap.get('machine'), 'd');
    t.assertEquals(rMap.get('1'), 'e');
    t.assertEquals(cMap.get('dog'), 'f');
    t.assertEquals(rMap.get('2'), 'g');
    t.assertEquals(rIdMap.get('doggo'), 'a');
});
t.test('deepNamespaces', function () {
    var r = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['id']);
    var p3 = r.namespace(['p1', 'p2', 'p3']);
    t.assertEquals(p3.namespaces.join('/'), 'root/p1/p2/p3');
    var p2 = p3.parent;
    t.assert(p2);
    t.assertEquals(p2.namespaces.join('/'), 'root/p1/p2');
    var p1 = p2.parent;
    t.assert(p1);
    t.assertEquals(p1.namespaces.join('/'), 'root/p1');
    var p0 = p1.parent;
    t.assert(p0);
    t.assertEquals(p0, r);
    t.assertEquals(p0.namespaces.join('/'), 'root');
});
t.test('pathSpecToParts', function () {
    t.assertArrayEquals(renamer_1.Renamer.pathSpecToParts('/'), ['/']);
    t.assertArrayEquals(renamer_1.Renamer.pathSpecToParts('dog'), ['dog']);
    t.assertArrayEquals(renamer_1.Renamer.pathSpecToParts('/dog'), ['/', 'dog']);
    t.assertArrayEquals(renamer_1.Renamer.pathSpecToParts('dog/cat'), ['dog', 'cat']);
    t.assertArrayEquals(renamer_1.Renamer.pathSpecToParts('../dog'), ['..', 'dog']);
    t.assertArrayEquals(renamer_1.Renamer.pathSpecToParts('dog/cat/parrot'), [
        'dog',
        'cat',
        'parrot',
    ]);
});
t.test('namespaceSansImports', function () {
    var r = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['id']);
    t.assertEquals(r.addName('id', 'dog'), 'a');
    var component = r.namespace('component');
    t.assertEquals(component.addName('id', 'dog'), 'b');
});
t.test('namespaceImportModuleToParent', function () {
    var r = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['id']);
    t.assertEquals(r.addName('id', 'dog'), 'a');
    var component = r.namespace('component');
    component["import"]('..', 'id', 'dog');
    t.assertEquals(component.addName('id', 'dog'), 'a');
});
t.test('namespaceImportParentToModule', function () {
    var r = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['id']);
    // Import before module even exists.
    r["import"]('component', 'id', 'dog');
    t.assertEquals(r.addName('id', 'dog'), 'a');
    var component = r.namespace('component');
    t.assertEquals(component.addName('id', 'dog'), 'a');
});
t.test('namespecBasic', function () {
    var spec = "\nnamespace slider\n\nfrom .. import\n  css\n    slider\n";
    var parser = new namespec_1.NamespecParser(spec);
    var _a = parser.parse(), namespace = _a.namespace, imports = _a.imports;
    t.assertEquals(namespace, 'slider');
    t.assertArrayEquals(Array.from(imports.keys()), ['..']);
    var map = imports.get('..');
    t.assertArrayEquals(Array.from(map.keys()), ['css']);
    t.assertArrayEquals(Array.from(map.values())[0], ['slider']);
});
t.test('namespecReserves', function () {
    var spec = "\nnamespace slider\n\nreserve\n  css\n    cat\n\nfrom .. import\n  css\n    slider\n";
    var parser = new namespec_1.NamespecParser(spec);
    var _a = parser.parse(), namespace = _a.namespace, imports = _a.imports, reserves = _a.reserves;
    t.assertEquals(namespace, 'slider');
    t.assertArrayEquals(Array.from(reserves.keys()), ['css']);
    t.assertArrayEquals(Array.from(Array.from(reserves.values())[0].values()), ['cat']);
    t.assertArrayEquals(Array.from(imports.keys()), ['..']);
    var map = imports.get('..');
    t.assertArrayEquals(Array.from(map.keys()), ['css']);
    t.assertArrayEquals(Array.from(map.values())[0], ['slider']);
});
t.test('namespecReservesAtEnd', function () {
    var spec = "\nnamespace slider\n\nfrom .. import\n  css\n    slider\n\nreserve\n  css\n    cat\n";
    var parser = new namespec_1.NamespecParser(spec);
    var _a = parser.parse(), namespace = _a.namespace, imports = _a.imports, reserves = _a.reserves;
    t.assertEquals(namespace, 'slider');
    t.assertArrayEquals(Array.from(imports.keys()), ['..']);
    var map = imports.get('..');
    t.assertArrayEquals(Array.from(map.keys()), ['css']);
    t.assertArrayEquals(Array.from(map.values())[0], ['slider']);
    t.assertArrayEquals(Array.from(reserves.keys()), ['css']);
    t.assertArrayEquals(Array.from(Array.from(reserves.values())[0].values()), ['cat']);
});
t.test('namespecMultipleValues', function () {
    var spec = "\nnamespace slider\n\nfrom .. import\n  css\n    slider\n    oven\n    mitten\n  id\n    dog\n    cat\n";
    var parser = new namespec_1.NamespecParser(spec);
    var _a = parser.parse(), namespace = _a.namespace, imports = _a.imports;
    t.assertEquals(namespace, 'slider');
    t.assertArrayEquals(Array.from(imports.keys()), ['..']);
    var map = imports.get('..');
    t.assertArrayEquals(Array.from(map.keys()), ['css', 'id']);
    t.assertArrayEquals(Array.from(map.values())[0], ['slider', 'oven', 'mitten']);
    t.assertArrayEquals(Array.from(map.values())[1], ['dog', 'cat']);
});
t.test('namespecMultipleImports', function () {
    var spec = "\nnamespace slider\n\nfrom .. import\n  css\n    dog\n    cat\n    bark\n  id\n    yes\nfrom /slider import\n  css\n    slider\n";
    var parser = new namespec_1.NamespecParser(spec);
    var _a = parser.parse(), namespace = _a.namespace, imports = _a.imports;
    t.assertEquals(namespace, 'slider');
    t.assertArrayEquals(Array.from(imports.keys()), ['..', '/slider']);
    var parentMap = imports.get('..');
    t.assertArrayEquals(Array.from(parentMap.keys()), ['css', 'id']);
    t.assertArrayEquals(Array.from(parentMap.values())[0], ['dog', 'cat', 'bark']);
    t.assertArrayEquals(Array.from(parentMap.values())[1], ['yes']);
    var sliderMap = imports.get('/slider');
    t.assertArrayEquals(Array.from(sliderMap.keys()), ['css']);
    t.assertArrayEquals(Array.from(sliderMap.values())[0], ['slider']);
});
t.test('namespecNoTrailingWhitespace', function () {
    var spec = "\nnamespace slider\n\nreserve\n  css\n    cat\n\nfrom .. import\n  css\n    slider";
    var parser = new namespec_1.NamespecParser(spec);
    var _a = parser.parse(), namespace = _a.namespace, imports = _a.imports, reserves = _a.reserves;
    t.assertEquals(namespace, 'slider');
    t.assertArrayEquals(Array.from(reserves.keys()), ['css']);
    t.assertArrayEquals(Array.from(Array.from(reserves.values())[0].values()), ['cat']);
    t.assertArrayEquals(Array.from(imports.keys()), ['..']);
    var map = imports.get('..');
    t.assertArrayEquals(Array.from(map.keys()), ['css']);
    t.assertArrayEquals(Array.from(map.values())[0], ['slider']);
});
t.test('simpleIncrementerBasic', function () {
    var r = new renamer_1.Renamer(incrementer_1.SimpleIncrementer, ['cls', 'id']);
    t.assertEquals(r.addName('cls', 'dog'), 'dog');
    t.assertEquals(r.addName('id', 'dog'), 'dog');
    // Adding name again just reuses old name.
    t.assertEquals(r.addName('cls', 'dog'), 'dog');
});
t.test('simpleIncrementerNamespace', function () {
    var r = new renamer_1.Renamer(incrementer_1.SimpleIncrementer, ['cls']);
    t.assertEquals(r.addName('cls', 'dog'), 'dog');
    var module = r.namespace('module');
    t.assertEquals(module.addName('cls', 'dog'), 'dog_1');
});
t.test('moduleIncrementerNamespace', function () {
    var r = new renamer_1.Renamer(incrementer_1.ModuleIncrementer, ['cls']);
    t.assertEquals(r.addName('cls', 'dog'), 'dog');
    var module = r.namespace('module');
    t.assertEquals(module.addName('cls', 'dog'), 'module_dog');
    var badModule = r.namespace('mod^ule');
    t.assertEquals(badModule.addName('cls', 'dog'), 'module_dog_1');
});
t.test('importWarning', function () {
    var r = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['cls']);
    r["import"]('module', 'cls', 'dog');
    var danglers = r.danglingImports();
    t.assertEquals(danglers.length, 1);
    t.assertObjectEquals(danglers[0], {
        sourceNamespace: 'root',
        importNamespace: 'root/module',
        type: 'cls',
        name: 'dog'
    });
});
t.test('importOkModuleToParent', function () {
    var r = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['id']);
    t.assertEquals(r.addName('id', 'dog'), 'a');
    var component = r.namespace('component');
    component["import"]('..', 'id', 'dog');
    // No dangling imports.
    t.assertEquals(r.danglingImports().length, 0);
});
t.test('importWarningParentToModule', function () {
    var r = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['id']);
    // Import before module even exists.
    r["import"]('component', 'id', 'dog');
    r.addName('id', 'dog');
    r.namespace('component');
    // Don't add id dog from component...
    // Some dangling imports.
    var danglers = r.danglingImports();
    t.assertEquals(danglers.length, 1);
    t.assertObjectEquals(danglers[0], {
        sourceNamespace: 'root',
        importNamespace: 'root/component',
        type: 'id',
        name: 'dog'
    });
});
t.test('importOkParentToModule', function () {
    var r = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['id']);
    // Import before module even exists.
    r["import"]('component', 'id', 'dog');
    r.addName('id', 'dog');
    var component = r.namespace('component');
    component.addName('id', 'dog');
    // No dangling imports.
    t.assertEquals(r.danglingImports().length, 0);
});
t.test('danglingImportsRecursive', function () {
    var root = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['id']);
    var r = root.namespace('root2');
    // Import before module even exists.
    r["import"]('component', 'id', 'dog');
    r.addName('id', 'dog');
    // const component = r.namespace('component');
    // Don't add id dog from component...
    // Some dangling imports.
    t.assertEquals(root.danglingImports().length, 1);
});
t.test('reservedIncrementer', function () {
    var m = new incrementer_1.MinimalIncrementer();
    m.reserve('b');
    t.assertEquals(m.next(), 'a');
    t.assertEquals(m.next(), 'c');
});
t.test('reservedRenamerChild', function () {
    var r = new renamer_1.Renamer(incrementer_1.SimpleIncrementer, ['id']);
    var component = r.namespace('component');
    component.reserve('id', 'dog');
    var dog = r.addName('id', 'dog');
    t.assertEquals(dog, 'dog_1');
});
t.test('reservedRenamerParent', function () {
    var r = new renamer_1.Renamer(incrementer_1.SimpleIncrementer, ['id']);
    var component = r.namespace('component');
    r.reserve('id', 'dog');
    var dog = component.addName('id', 'dog');
    t.assertEquals(dog, 'dog_1');
});
t.test('virtualFsBasic', function () {
    var fs = new virtual_fs_1.VirtualFs();
    fs.writeFileSync('/hello.txt', 'hello world');
    t.assertEquals(fs.readFileSync('/hello.txt').toString(), 'hello world');
    t.assertArrayEquals(fs.readdirSync('/'), ['hello.txt']);
});
t.test('virtualFsListDir', function () {
    var fs = new virtual_fs_1.VirtualFs();
    fs.writeFileSync('/src/index.html', '');
    fs.writeFileSync('/src/style.css', '');
    fs.writeFileSync('/src/component/component.html', '');
    fs.writeFileSync('/src/component/component.css', '');
    fs.writeFileSync('/src/component/toggle/toggle.css', '');
    fs.writeFileSync('/src/component/toggle/tests/style/test.js', '');
    fs.writeFileSync('/out/module/component/test.txt', '');
    t.assertArrayEquals(fs.readdirSync('/'), ['out/', 'src/']);
    t.assertArrayEquals(fs.readdirSync('/src/'), [
        'component/',
        'index.html',
        'style.css',
    ]);
    t.assertArrayEquals(fs.readdirSync('/src/component/'), [
        'component.css',
        'component.html',
        'toggle/',
    ]);
    t.assertArrayEquals(fs.readdirSync('/src/component/toggle/'), [
        'tests/',
        'toggle.css',
    ]);
    t.assertArrayEquals(fs.readdirSync('/src/component/toggle/tests/'), ['style/']);
    t.assertArrayEquals(fs.readdirSync('/src/component/toggle/tests/style/'), [
        'test.js',
    ]);
    t.assertArrayEquals(fs.readdirSync('/out/'), ['module/']);
    t.assertArrayEquals(fs.readdirSync('/out/module/'), ['component/']);
    t.assertArrayEquals(fs.readdirSync('/out/module/component/'), ['test.txt']);
});
t.test('distinguisherBasic', function () {
    var fs = new virtual_fs_1.VirtualFs();
    fs.writeFileSync('/src/style.css', '._cls-content { color: gray; }');
    var config = {
        inputDir: '/src/',
        outputDir: '/out/',
        incrementer: 'minimal',
        types: ['cls', 'id'],
        exclude: []
    };
    var d = new distinguisher_1.Distinguisher(config, fs, fs.dirname, false);
    d.run();
    t.assertArrayEquals(fs.readdirSync('/'), ['out/', 'src/']);
    t.assertArrayEquals(fs.readdirSync('/out/'), ['style.css']);
    t.assertEquals(fs.readFileSync('/out/style.css').toString(), '.a { color: gray; }');
});
t.test('distinguisherClsAndIdBasic', function () {
    var fs = new virtual_fs_1.VirtualFs();
    fs.writeFileSync('/src/index.html', '<div id="_id-content" class="_cls-content">Content here</div>');
    fs.writeFileSync('/src/component/style.css', "\n    ._cls-content {\n      color: gray;\n    }\n    #_id-content {\n      font-weight: bold;\n    }");
    var config = {
        inputDir: '/src/',
        outputDir: '/out/',
        incrementer: 'minimal',
        types: ['cls', 'id'],
        exclude: []
    };
    var d = new distinguisher_1.Distinguisher(config, fs, fs.dirname, false);
    d.run();
    t.assertEquals(fs.readFileSync('/out/index.html').toString(), '<div id="a" class="a">Content here</div>');
    t.assertEquals(fs.readFileSync('/out/component/style.css').toString(), "\n    .a {\n      color: gray;\n    }\n    #a {\n      font-weight: bold;\n    }");
});
function toggleSrcIndex(toggleClass) {
    return "\n<link rel=\"stylesheet\" href=\"toggle/toggle.css\">\n\n<style>\n  ." + toggleClass + " {\n    background: green;\n  }\n</style>\n\n<div class=\"" + toggleClass + "\"></div>\n\n<script src=\"toggle/toggle.js\"></script>";
}
function toggleToggleCss(toggleClass) {
    return "\n  ." + toggleClass + " {\n    background: blue;\n  }";
}
function toggleToggleJs(toggleClass) {
    return "\n  // Create a toggle element from scratch.\n  const div = document.createElement('div');\n  div.classList.add('" + toggleClass + "');\n  \n  // Code to render it.\n  ...";
}
function toggleTest(initialClass, indexClass, cssClass, jsClass, playWithFiles) {
    var fs = new virtual_fs_1.VirtualFs();
    fs.writeFileSync('/src/index.html', toggleSrcIndex(initialClass));
    fs.writeFileSync('/src/toggle/toggle.css', toggleToggleCss(initialClass));
    fs.writeFileSync('/src/toggle/toggle.js', toggleToggleJs(initialClass));
    if (playWithFiles != null)
        playWithFiles(fs);
    var config = {
        inputDir: '/src/',
        outputDir: '/out/',
        incrementer: 'minimal',
        types: ['cls', 'id'],
        exclude: []
    };
    var d = new distinguisher_1.Distinguisher(config, fs, fs.dirname, false);
    d.run();
    t.assertEquals(fs.readFileSync('/out/index.html').toString(), toggleSrcIndex(indexClass));
    t.assertEquals(fs.readFileSync('/out/toggle/toggle.css').toString(), toggleToggleCss(cssClass));
    t.assertEquals(fs.readFileSync('/out/toggle/toggle.js').toString(), toggleToggleJs(jsClass));
}
t.test('distinguisherNamespaceClash', function () {
    toggleTest('_cls-toggle', 'a', 'a', 'a');
});
t.test('distinguisherNamespaceFixComponent', function () {
    toggleTest('_cls-toggle', 'a', 'b', 'b', function (fs) {
        fs.writeFileSync('/src/toggle/.namespec', 'namespace toggle');
    });
});
t.test('distinguisherNamespaceImport', function () {
    toggleTest('_cls-toggle', 'a', 'a', 'a', function (fs) {
        fs.writeFileSync('/src/toggle/.namespec', "namespace toggle\n\nfrom .. import\n  cls\n    toggle");
    });
});
t.test('distinguisherNamespecReserve', function () {
    toggleTest('_cls-toggle', 'b', 'c', 'c', function (fs) {
        fs.writeFileSync('/src/toggle/.namespec', "namespace toggle\n\nreserve\n  cls\n    a");
    });
});
function cli(argsString, fsInit) {
    var fs = new virtual_fs_1.VirtualFs();
    if (fsInit != null)
        fsInit(fs);
    return new cli_1.CLI(null, argsString.split(' ').filter(function (x) { return x.trim().length > 0; }), fs, fs.require.bind(fs), fs.dirname, fs.join, function () { return '/'; }, true).process();
}
t.test('distinguishCliShowDialogs', function () {
    // Help
    t.assertObjectEquals(cli('--help'), { showUsage: 'base' });
    t.assertObjectEquals(cli('-h'), { showUsage: 'base' });
    t.assertObjectEquals(cli('--usage'), { showUsage: 'base' });
    t.assertObjectEquals(cli('help'), { showUsage: 'base' });
    t.assertObjectEquals(cli('help init'), { showUsage: 'init' });
    t.assertObjectEquals(cli('help rename'), { showUsage: 'rename' });
    t.assertObjectEquals(cli('--invalid-option'), { showUsage: 'base' });
    t.assertObjectEquals(cli('rename --invalid-option'), { showUsage: 'rename' });
    // Version
    t.assertObjectEquals(cli('-v'), { showVersion: true });
    t.assertObjectEquals(cli('--version'), { showVersion: true });
    // Splash
    t.assertObjectEquals(cli('--splash'), { showSplash: true });
});
t.test('distinguishCliInit', function () {
    t.assertObjectEquals(cli('init'), { initFn: 'distinguish.config.js' });
    t.assertObjectEquals(cli('init config.js'), { initFn: 'config.js' });
    t.assertObjectEquals(cli('init -o'), { showUsage: 'init' });
    t.assertObjectEquals(cli('init config.js extraneous'), { showUsage: 'init' });
});
t.test('distinguishCliRenameNoConfig', function () {
    var result = cli('rename');
    // File won't exist yet.
    t.assertEquals(result.showUsage, 'rename');
});
function getConfig(incrementer, types, inputDir, outputDir, exclude) {
    if (incrementer === void 0) { incrementer = 'simple'; }
    if (types === void 0) { types = ['cls', 'id']; }
    if (inputDir === void 0) { inputDir = 'src/'; }
    if (outputDir === void 0) { outputDir = 'out/'; }
    if (exclude === void 0) { exclude = []; }
    return "exports.default = {\n    incrementer: '" + incrementer + "', // the incrementer to use ([minimal, simple, module])\n    types: " + JSON.stringify(types) + ", // the types to rename (e.g. CSS classes, IDs)\n  \n    inputDir: '" + inputDir + "', // the input directory to use\n    outputDir: '" + outputDir + "', // the output directory to use\n  \n    exclude: " + JSON.stringify(exclude) + ", // a regular expression array describing files to exclude from renaming\n  };\n  ";
}
t.test('distinguishCliRenameWithConfig', function () {
    var e_1, _a;
    try {
        for (var _b = __values([
            'rename ',
            'rename -c',
            'rename -c distinguish.config.js',
            'rename --config',
            'rename --config distinguish.config.js',
        ]), _c = _b.next(); !_c.done; _c = _b.next()) {
            var cmd = _c.value;
            var result = cli(cmd, function (fs) {
                fs.writeFileSync('distinguish.config.js', getConfig());
            });
            // Let's check we get a matching config.
            t.assert(result.opts);
            var opts = result.opts;
            t.assertEquals(opts, {
                configFile: 'distinguish.config.js',
                incrementer: 'simple',
                types: ['cls', 'id'],
                inputDir: 'src/',
                outputDir: 'out/',
                exclude: []
            });
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b["return"])) _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
});
t.test('distinguishCliRenameWithConfigDifferentFile', function () {
    var e_2, _a;
    try {
        for (var _b = __values([
            'rename -c specification/config.js',
            'rename --config specification/config.js',
        ]), _c = _b.next(); !_c.done; _c = _b.next()) {
            var cmd = _c.value;
            var result = cli(cmd, function (fs) {
                fs.writeFileSync('specification/config.js', getConfig());
            });
            // Let's check we get a matching config.
            t.assert(result.opts);
            var opts = result.opts;
            t.assertEquals(opts, {
                configFile: 'specification/config.js',
                incrementer: 'simple',
                types: ['cls', 'id'],
                inputDir: 'src/',
                outputDir: 'out/',
                exclude: []
            });
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b["return"])) _a.call(_b);
        }
        finally { if (e_2) throw e_2.error; }
    }
});
t.test('distinguishCliRenameWithOverrides', function () {
    var result = cli('rename -c spec/settings/config.js -n minimal -t dog,cat,hen -i input/src/ -o dest/ -e dog,cat', function (fs) {
        fs.writeFileSync('spec/settings/config.js', getConfig());
    });
    // Let's check we get a matching config.
    t.assert(result.opts);
    var opts = result.opts;
    t.assertEquals(opts, {
        configFile: 'spec/settings/config.js',
        incrementer: 'minimal',
        types: ['dog', 'cat', 'hen'],
        inputDir: 'input/src/',
        outputDir: 'dest/',
        exclude: [/dog/, /cat/]
    });
});
t.test('distinguishCliWatch', function () {
    var result = cli('rename -c spec/settings/config.js -w', function (fs) {
        fs.writeFileSync('spec/settings/config.js', getConfig());
    });
    // Let's check we get a matching config.
    t.assert(result.opts);
    var opts = result.opts;
    t.assertEquals(opts, {
        configFile: 'spec/settings/config.js',
        incrementer: 'simple',
        types: ['cls', 'id'],
        inputDir: 'src/',
        outputDir: 'out/',
        exclude: [],
        watch: true
    });
});
