"use strict";
exports.__esModule = true;
/**
 * Returns the directory name for a file. Do not depend on fs so we can play with this
 * library in the browser.
 * @param fn The file name
 */
function getDir(fn) {
    var addSlashLater = false;
    if (fn.endsWith('/')) {
        fn = fn.substr(0, fn.length - 1);
        addSlashLater = true;
    }
    var parts = fn.split('/');
    return {
        directory: parts.slice(0, parts.length - 1).join('/') + '/',
        tail: parts[parts.length - 1] + (addSlashLater ? '/' : '')
    };
}
var VirtualFs = /** @class */ (function () {
    function VirtualFs() {
        this.directories = new Map();
        this.fileMap = new Map();
    }
    VirtualFs.prototype.dirname = function (path) {
        return getDir(path).directory;
    };
    VirtualFs.prototype.join = function () {
        var parts = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            parts[_i] = arguments[_i];
        }
        var trailingSlash = parts[parts.length - 1].endsWith('/');
        return (parts.map(function (x) { return (x.endsWith('/') ? x.substr(0, x.length - 1) : x); }).join('/') +
            (trailingSlash ? '/' : ''));
    };
    VirtualFs.prototype.require = function (path) {
        var contents = this.readFileSync(path).toString();
        var sandbox = {};
        var wrappedCode = "\n      void function(exports) {\n        " + contents + "\n      }(sandbox)";
        eval(wrappedCode);
        return sandbox;
    };
    VirtualFs.prototype.setDirectory = function (fnOrDir) {
        // Base case.
        if (fnOrDir == '/')
            return;
        // Create directories recursively.
        this.mkdirSync(fnOrDir);
    };
    VirtualFs.prototype.statSync = function (fn) {
        // Pretty naive directory checking.
        if (fn.endsWith('/')) {
            return {
                isDirectory: function () {
                    return true;
                }
            };
        }
        return {
            isDirectory: function () {
                return false;
            }
        };
    };
    VirtualFs.prototype.mkdirSync = function (dir) {
        var _a = getDir(dir), parentDir = _a.directory, tail = _a.tail;
        var directorySet = this.directories.get(parentDir);
        if (directorySet != null) {
            directorySet.add(tail);
        }
        else {
            this.directories.set(parentDir, new Set([tail]));
        }
        this.setDirectory(parentDir);
    };
    VirtualFs.prototype.writeFileSync = function (fn, contents) {
        this.fileMap.set(fn, contents);
        this.setDirectory(fn);
    };
    VirtualFs.prototype.existsSync = function (fn) {
        if (!fn.endsWith('/')) {
            // File existence check.
            return this.fileMap.has(fn);
        }
        else {
            if (fn == '/')
                return true; // root always exists.
            // Check directory existence.
            var _a = getDir(fn), parentDir = _a.directory, tail = _a.tail;
            var directorySet = this.directories.get(parentDir);
            if (directorySet != null)
                return directorySet.has(tail);
            return false;
        }
    };
    VirtualFs.prototype.readFileSync = function (fn) {
        var contents = this.fileMap.get(fn);
        return {
            toString: function () {
                return contents;
            }
        };
    };
    VirtualFs.prototype.readdirSync = function (dir) {
        if (!dir.endsWith('/'))
            dir += '/';
        var directorySet = this.directories.get(dir);
        if (directorySet == null)
            return [];
        return Array.from(directorySet.values()).sort();
    };
    return VirtualFs;
}());
exports.VirtualFs = VirtualFs;
