"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var e_1, _a, e_2, _b, e_3, _c;
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var magic_string_1 = __importDefault(require("magic-string"));
var renamer_1 = require("./renamer");
var namespec_1 = require("./namespec");
var incrementer_1 = require("./incrementer");
var log_1 = require("./log");
var NAMESPEC = '.namespec';
function walkSync(dir, outDir, renamer, filelist) {
    if (dir === void 0) { dir = ''; }
    if (outDir === void 0) { outDir = ''; }
    if (filelist === void 0) { filelist = []; }
    var e_4, _a, e_5, _b, e_6, _c, e_7, _d, e_8, _e;
    if (!dir.endsWith('/'))
        dir += '/';
    if (!outDir.endsWith('/'))
        outDir += '/';
    var namespecPath = "" + dir + NAMESPEC;
    if (fs_1["default"].existsSync(namespecPath)) {
        // Parse a namespec file to determine the renamer's new scope.
        var namespec = new namespec_1.NamespecParser(fs_1["default"].readFileSync(namespecPath).toString()).parse();
        // Set the namespace.
        renamer = renamer.namespace(renamer_1.Renamer.pathSpecToParts(namespec.namespace));
        try {
            // Set imports.
            for (var _f = __values(namespec.imports.entries()), _g = _f.next(); !_g.done; _g = _f.next()) {
                var _h = __read(_g.value, 2), importName = _h[0], importMap = _h[1];
                try {
                    for (var _j = __values(importMap.entries()), _k = _j.next(); !_k.done; _k = _j.next()) {
                        var _l = __read(_k.value, 2), type = _l[0], names = _l[1];
                        try {
                            for (var names_1 = __values(names), names_1_1 = names_1.next(); !names_1_1.done; names_1_1 = names_1.next()) {
                                var name_1 = names_1_1.value;
                                renamer["import"](importName, type, name_1);
                            }
                        }
                        catch (e_6_1) { e_6 = { error: e_6_1 }; }
                        finally {
                            try {
                                if (names_1_1 && !names_1_1.done && (_c = names_1["return"])) _c.call(names_1);
                            }
                            finally { if (e_6) throw e_6.error; }
                        }
                    }
                }
                catch (e_5_1) { e_5 = { error: e_5_1 }; }
                finally {
                    try {
                        if (_k && !_k.done && (_b = _j["return"])) _b.call(_j);
                    }
                    finally { if (e_5) throw e_5.error; }
                }
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (_g && !_g.done && (_a = _f["return"])) _a.call(_f);
            }
            finally { if (e_4) throw e_4.error; }
        }
        try {
            // Set reserves.
            for (var _m = __values(namespec.reserves.entries()), _o = _m.next(); !_o.done; _o = _m.next()) {
                var _p = __read(_o.value, 2), type = _p[0], reserves = _p[1];
                try {
                    for (var _q = __values(reserves.values()), _r = _q.next(); !_r.done; _r = _q.next()) {
                        var reserveName = _r.value;
                        renamer.reserve(type, reserveName);
                    }
                }
                catch (e_8_1) { e_8 = { error: e_8_1 }; }
                finally {
                    try {
                        if (_r && !_r.done && (_e = _q["return"])) _e.call(_q);
                    }
                    finally { if (e_8) throw e_8.error; }
                }
            }
        }
        catch (e_7_1) { e_7 = { error: e_7_1 }; }
        finally {
            try {
                if (_o && !_o.done && (_d = _m["return"])) _d.call(_m);
            }
            finally { if (e_7) throw e_7.error; }
        }
    }
    var files = fs_1["default"].readdirSync(dir == '' ? '.' : dir);
    files.forEach(function (file) {
        var fn = "" + dir + file;
        if (fs_1["default"].statSync(fn).isDirectory()) {
            filelist = walkSync(fn + "/", "" + outDir + file, renamer, filelist);
        }
        else {
            filelist.push({
                inputFile: "" + dir + file,
                outputFile: "" + outDir + file,
                renamer: renamer
            });
        }
    });
    return filelist;
}
var INPUT = 'concept';
var OUTPUT = 'concept_out';
var types = ['cls', 'id'];
var rootRenamer = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, types);
// Adapted from https://stackoverflow.com/a/34509653
function ensureDirectoryExistence(filePath) {
    var dirname = path_1["default"].dirname(filePath);
    if (fs_1["default"].existsSync(dirname))
        return;
    ensureDirectoryExistence(dirname);
    fs_1["default"].mkdirSync(dirname);
}
function getAllMatches(regex, str) {
    var result;
    var results = [];
    while ((result = regex.exec(str)) !== null) {
        results.push(result);
    }
    return results;
}
var startTime = Date.now();
try {
    for (var _d = __values(walkSync('concept', 'concept_out', rootRenamer)), _e = _d.next(); !_e.done; _e = _d.next()) {
        var _f = _e.value, inputFile = _f.inputFile, outputFile = _f.outputFile, renamer = _f.renamer;
        var contentsString = fs_1["default"].readFileSync(inputFile).toString();
        var contents = new magic_string_1["default"](contentsString);
        var hadMatches = false;
        try {
            for (var types_1 = __values(types), types_1_1 = types_1.next(); !types_1_1.done; types_1_1 = types_1.next()) {
                var type = types_1_1.value;
                // Find all matches.
                var matches = getAllMatches(new RegExp("_(" + type + ")-([a-zA-Z0-9_-]+)", 'g'), contentsString);
                for (var i = 0; i < matches.length; i++) {
                    hadMatches = true; // there was at least a match somewhere
                    var _g = __read(matches[i], 3), fullMatch = _g[0], typeMatch = _g[1], name_2 = _g[2];
                    var index = matches[i].index;
                    var renamed = renamer.addName(typeMatch, name_2);
                    contents.overwrite(index, index + fullMatch.length, renamed);
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (types_1_1 && !types_1_1.done && (_b = types_1["return"])) _b.call(types_1);
            }
            finally { if (e_2) throw e_2.error; }
        }
        if (hadMatches) {
            log_1.logStyle(log_1.STATUS, "Writing " + outputFile + " with namespace " + renamer.namespaces.join('/'));
        }
        ensureDirectoryExistence(outputFile);
        fs_1["default"].writeFileSync(outputFile, contents.toString());
    }
}
catch (e_1_1) { e_1 = { error: e_1_1 }; }
finally {
    try {
        if (_e && !_e.done && (_a = _d["return"])) _a.call(_d);
    }
    finally { if (e_1) throw e_1.error; }
}
var overallTime = Date.now() - startTime;
log_1.logStyle(log_1.BOLD, "\nWrote output in " + overallTime / 1000 + "s");
var danglers = rootRenamer.danglingImports();
if (danglers.length > 0)
    console.log('\n');
try {
    for (var danglers_1 = __values(danglers), danglers_1_1 = danglers_1.next(); !danglers_1_1.done; danglers_1_1 = danglers_1.next()) {
        var _h = danglers_1_1.value, sourceNamespace = _h.sourceNamespace, importNamespace = _h.importNamespace, type = _h.type, name_3 = _h.name;
        log_1.logStyle(log_1.WARN, "Dangling import: " + sourceNamespace + " imports unused {type: " + type + ", name: " + name_3 + "} from " + importNamespace);
    }
}
catch (e_3_1) { e_3 = { error: e_3_1 }; }
finally {
    try {
        if (danglers_1_1 && !danglers_1_1.done && (_c = danglers_1["return"])) _c.call(danglers_1);
    }
    finally { if (e_3) throw e_3.error; }
}
